@@ -22,6 +22,7 @@ from pyg_base._dates import dt
 from pyg_base._txt import lower
 import pandas as pd
 import numpy as np
+from functools import partial
 from copy import copy
 import inspect
 import datetime
@@ -1156,14 +1157,20 @@ def le_(a, b, join = 'ij', method = None, columns = 'ij'):
 
 def _align_columns(a, b, func):
     da, db = [len(getattr(x, 'shape', ())) for x in (a,b)]
-    if da == db or max(da, db) <= 1:
+    if min(da, db) == 0:
         return func(a,b)
-    a, b = (a, b) if da == 2 else (b, a)
-    if is_series(b):
+    if da == 2 and db == 2 and list(a.columns) == list(b.columns): # perfect fit, no need to run as_series
+        return func(a,b)        
+    a,b = as_series(a), as_series(b)
+    da, db = [len(getattr(x, 'shape', ())) for x in (a,b)]
+    if da == db:       # (1,1) and (2,2)
+        return func(a,b)
+    if is_series(b):   # (2,1)
         b = pd.concat([b] * a.shape[1], axis=1)
         b.columns = a.columns
-    else:
-        b = np.concatenate([b] * a.shape[1], axis=1)
+    elif is_series(a): # (1,2)
+        a = pd.concat([a] * b.shape[1], axis=1)
+        a.columns = b.columns
     return func(a,b)
         
 def _minimum(a, b):
@@ -1367,6 +1374,9 @@ def df_slice(df, lb = None, ub = None, openclose = '(]', n = 1):
         [,] or c : close
         (,) or o : open
         ' ' : do not cut
+    n : int
+        This parameter is used when multiple timeseries are provided. 
+        n determines the number of columns stitched together.
     
     :Returns:
     -------
@@ -1504,7 +1514,8 @@ def df_unslice(df, ub):
     return dict(rs['u', 'ts'])
 
 
-def ts_gap(ts, gap = 'days'):
+
+def ts_gap(ts, gap = 'days', recent = True):
     """
     Calculate the gap going FORWARD in time. The last gap is measured against today
     
@@ -1537,11 +1548,14 @@ def ts_gap(ts, gap = 'days'):
     if gap == 'days':
         days = [(t1-t0).days for t0, t1 in zip(ts[:-1], ts[1:])]
         now = dt(0)
-        if ts[-1] <= now:
-            days.append((dt(0) - ts[-1]).days)
+        if recent:
+            if ts[-1] <= now:
+                days.append((dt(0) - ts[-1]).days)
+            else:
+                days.append(min(days))
+            return pd.Series(days, index = ts)
         else:
-            days.append(min(days))
-        return pd.Series(days, index = ts)
+            return pd.Series(days, index = ts[:-1])
     else:
         raise ValueError(f'pyg_base.ts_gap with gap={gap} not supported')
 
@@ -1575,7 +1589,7 @@ def ts_deal_with_issue(ts, issue_calc, issue_level = 0, deal = 'last'):
     else:
         raise ValueError(f'no idea how to deal with issues {issues} using {deal}')
 
-def ts_degap(ts, max_gap = 0, deal = 'last'):
+def ts_degap(ts, max_gap = 0, deal = 'last', recent = False):
     """
     Parameters
     ----------
@@ -1591,11 +1605,18 @@ def ts_degap(ts, max_gap = 0, deal = 'last'):
     -------
     timeseries
         a timeseries with gap issues handled
+    
 
+    :Example: gaps in recent values do not matter by default
+    ---------
+    >>> from pyg import *
+    >>> ts = pd.Series(1, drange(-1000,-500)) ## old data
+    >>> assert eq(ts_degap(ts, 10), ts)
+    >>> assert len(ts_degap(ts, 10, recent = True)) == 0 # There is no recent data
     """
     
-    return ts_deal_with_issue(ts, issue_calc = ts_gap, issue_level = max_gap, deal = deal)
-    
+    return ts_deal_with_issue(ts, issue_calc = partial(ts_gap, recent = recent), issue_level = max_gap, deal = deal)
+   
 
 def df_drop_index_duplicates(df, keep = 'last'):
     """
