@@ -1,11 +1,8 @@
 # -*- coding: utf-8 -*-
 import numpy as np
-from pyg_base._as_list import as_list
-from pyg_base._encode import as_primitive
-from pyg_base._types import is_nan, is_iterable, is_float, is_int, is_bool, is_str
+from pyg_base._types import is_nan, is_iterable
 from pyg_base._loop import len0
-from pyg_base._eq import veq
-
+from pyg_base._as_primitive import as_primitive
 
 __all__ = ['Cmp', 'cmp', 'sort',]
 
